%
% cdsim.m   
% This program computes empirical rejection rates of the J test for
% correctly specified and misspecified SDF models with 3 useful and  
% 2 spurious factors (Table I, Panel D, in the paper).
% The program can be easily modified to the setups in Panels A, B, and C.
% Note that the dimension K in the paper corresponds to K+1 in this code.
%
clear all
load monthly;     % Data for returns and factors: monthly, 1959:2-2012:12
R = 1+R25;        % 25 Fama-French gross returns
[T,N] = size(R);
F = [vw smb hml]; % market, SMB, and HML factors
K1 = size(F,2);   % number of useful factors
K2 = 2;           % number of spurious factors
P = null(ones(1,N));
V = cov([F R],1);
mu1 = mean(F)';
mu2 = mean(R)';
V11 = V(1:K1,1:K1);
V22 = V(K1+1:end,K1+1:end);
V21 = V(K1+1:end,1:K1);
for mis=1:-1:0; % 1 for misspecified model; 0 for correctly specified model
r = K2+1-mis;   % null space of H
if mis==0;
    b = V21*inv(V11);
    X = [ones(N,1) b];
    mu2 = X*inv(X'*X)*X'*mu2; % imposing the restrictions of the model
end
mu = [mu1; mu2]; % mu is vector of means for generated factors and returns
Vh = sqrtm(V);   % V is covariance matrix for generated factors and returns  
TT = [200 600 1000];
maxT = max(TT);
nT = length(TT);
nobs = 100000;
cdtest = zeros(nobs,nT);
for i=1:nobs
    if rem(i,1000)==0
        disp(i)
    end
    Ys = ones(maxT,1)*mu'+randn(maxT,N+K1)*Vh;
    Fs = [Ys(:,1:K1) randn(maxT,K2)];
    Rs = Ys(:,K1+1:end);  
    for j=1:nT
        T = TT(j);
        F = Fs(1:T,:);
        R = Rs(1:T,:);
        [cdtest(i,j),~,~,~,~,~,~,~,~,~] = cugmm(R,F);
    end
end
K = K1+K2;   % K is the number of factors (useful+spurious)
p = [0.01 0.05 0.1 0.25 0.5 0.75 0.9 0.95 0.99];
alpha = [0.1 0.05 0.01];
cutoff1 = chi2inv(1-alpha,N-K-1);
if mis==1;
    delete cdsim_m.out
    diary cdsim_m.out
    fprintf(' Misspecified Model with 3 Useful and 2 Spurious Factors\n')
elseif mis==0;
    delete cdsim_c.out
    diary cdsim_c.out
    fprintf(' Correctly Specified Model with 3 Useful and 2 Spurious Factors\n')
end
fprintf(' Number of Simulations = %6.0f\n',nobs)
fprintf(' N = %2.0f,  K = %1.0f\n',N,K+1)
fprintf(' Null Space of H = %1.0f\n',r)
fprintf('\n Distribution of CD Test:\n')
fprintf('   T     Mean     Std     1%%      5%%     10%%     25%%     50%%     75%%     90%%     95%%     99%%\n')
for i=1:nT
    fprintf('%5.0f %8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f\n', ...
             TT(i),mean(cdtest(:,i)),std(cdtest(:,i)),prctile(cdtest(:,i),100*p))
end
[mulr,mu2lr] = wisheigminmom(r,N-K-2+r);
stdlr = sqrt(mu2lr-mulr^2);
fprintf('Limit %8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f\n',mulr,stdlr,wisheigminicdf(p,r,N-K-2+r))
fprintf('\n Rejection rates of CD Test:\n')
fprintf('   T       10%%       5%%       1%%      \n') 
for j=1:nT
    fprintf(' %4.0f   %8.6f  %8.6f  %8.6f\n',...
    TT(j),mean(cdtest(:,j)>cutoff1(1)),mean(cdtest(:,j)>cutoff1(2)),mean(cdtest(:,j)>cutoff1(3)))
end
fprintf('Limit   %8.6f  %8.6f  %8.6f\n',1-wisheigmincdf(cutoff1,r,N-K-2+r))
diary off
end
