%
% cugmm.m
% This function estimates a linear SDF model using continuously-updated GMM
% Inputs: 
%   R: a TxN matrix of random payoffs of N assets (typically
%      gross returns)
%   F: a Tx(K-1) matrix of factors, K is the number of SDF parameters
%   q: cost of the N assets (default = 1_N)
%   nlag: number of lags for the Newey-West (default = 0)
% Outputs:
%   Jtest: over-identification test statistic
%   pval: p-value of the J-test
%   lam: estimate of lambda
%   tratc: t-ratios of estimated lambda assuming the model
%          is correctly specified
%   tratm: t-ratios of estimated lambda assuming the model
%          is potentially misspecified  
%   muf: estimate of mean of the factors
%   Vf: estimate of covariance matrix of the factors
%   B: estimate of beta of the factors
%   gam: estimate of zero-beta rate and risk premia parameters
%   muh: estimate of expected return
%
function [Jtest,pval,lam,tratc,tratm,muf,Vf,B,gam,muh,chat] = cugmm(R,F,q,nlag)
[T,N] = size(R);
if nargin<4
   nlag = 0;
   if nargin<3
      q = ones(N,1);
   end
end
X = [ones(T,1) F];
K = size(X,2);
P = null(q');
R1 = R*P;
D = (R1'*X)/T;
m = kron(X,ones(1,N-1)).*repmat(R1,1,K);  
m = m-ones(T,1)*mean(m);
Vd = (m'*m)/T;
for i=1:nlag
    Vd1 = (m(i+1,T,:)'*m(1:T-i))/(T-i);
    Vd = Vd+(1-i/(lag+1))*(Vd1+Vd1');
end
%
%   Obtain initial estimate of lam for the rank test
%
V = cov([F R1],1);
XX = (X'*X)/T;
V11 = V(1:K-1,1:K-1);
V12 = V(1:K-1,K:end);
V21 = V12';
V22 = V(K:end,K:end); 
S = V22-V21*inv(V11)*V12;
[v1,d1] = eig(D'*inv(S)*D,XX);
[~,ind] = min(diag(d1));
c0 = v1(:,ind);
c0 = c0/sqrt(c0'*c0);
options = optimset('DerivativeCheck','off','LargeScale','on','Display','notify','Algorithm','interior-point', ....
          'GradObj','on','GradConstr','on','TolFun',1e-10,'TolX',1e-10,'TolCon',1e-10,'MaxFunEvals',10000);
H = zeros(N-1,K);
H(:,1) = 1:N:(N-1)^2;
for i=2:K
    H(:,i) = (N-1)^2+H(:,i-1);
end
[chat,lmstat,exitflag,output]  = fmincon(@lmc,c0,[],[],[],[],[],[],@constr,options);
Jtest = T*lmstat;
pval = gammainc(Jtest/2,(N-K)/2,'upper');
if nargout>2
   D = (R'*X)/T;
   e = R.*(X*chat*ones(1,N));
   e = e-ones(T,1)*mean(e);
   Ve = (e'*e)/T;
   for i=1:nlag
       Ve1 = (e(i+1,T,:)'*e(1:T-i))/(T-i);
       Ve = Ve+(1-i/(lag+1))*(Ve1+Ve1');
   end
%
%  Normalize lambda so that it minimizes the CU-GMM objective function
%
   Vei = inv(Ve);
   cstar = (q'*Vei*q)/(q'*Vei*D*chat);   
   lam = cstar*chat;
   Vei = Vei/cstar^2;
%
%  standard error under correctly specified model
%
   Vlamc = inv(D'*Vei*D);
   tratc = sqrt(T)*lam./sqrt(diag(Vlamc));
end
%
%  Misspecification-robust standard error
%  (see Gospodinov, Kan, and Robotti, forthcoming in Econometric Reviews)
%
if nargout>4
   e1 = R.*(X*lam*ones(1,N))-ones(T,1)*q';
   ebar = mean(e1)';
   e = e1-ones(T,1)*ebar';
   gam1 = -Vei*ebar;
   c = 1+e*gam1;
   Gt = (R*gam1*ones(1,K)).*X; 
   Xc = X.*(c*ones(1,K));
   Gt1 = Gt-ones(T,1)*(gam1'*D);
   B = (R'*Xc)/T+(e'*Gt)/T;
   C = (Gt1'*Gt1)/T;
   H = inv(C-B'*Vei*B);
   ht = (Gt.*(c*ones(1,K))-(e.*(c*ones(1,N)))*Vei*B)*H;
   Vlamm = (ht'*ht)/T;
   for i=1:nlag
       Vlam1 = (ht(i+1,T,:)'*ht(1:T-i))/(T-i);
       Vlamm  = Vlamm+(1-i/(nlag+1))*(Vlam1+Vlam1');
   end
   tratm = sqrt(T)*lam./sqrt(diag(Vlamm));
end
%
%   Obtain other parameters for the beta-pricing representation
%
if nargout>5
   w = (1-e*Vei*(D*lam-q))/T;
   muf = F'*w;
%
%   Enlarge the system to obtain Vf and beta
%
   F1 = (F-ones(T,1)*muf').*(w*ones(1,K-1));
   Vf = (F'*F1);
   B = (R'*F1)*inv(Vf);
   denom = lam(1)+muf'*lam(2:end);
   gam = [1; -Vf*lam(2:end)]/denom;
   muh = [q B]*gam;
end

function [lmv,lmder] = lmc(c)
   l = D*c;
   c1 = kron(c',eye(N-1));
   Lambdail = inv(c1*Vd*c1')*l;       
   lmv = l'*Lambdail;
   h = kron(Lambdail'*c1*Vd,Lambdail');
   lmder = 2*Lambdail'*D-2*sum(h(H));
end

function [cc,ceq,ccd,ceqd] = constr(c)
   cc = [];
   ccd = [];
   ceq = c'*c-1;
   ceqd = 2*c;
end

end
